@@ -25,11 +25,16 @@ import com.beust.jcommander.ParameterException;
 import com.beust.jcommander.Parameters;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.ImmutableSet;
+import java.io.FileInputStream;
+import java.io.InputStream;
+import java.util.List;
+import java.util.Properties;
 import java.util.Set;
 import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.conf.Configurable;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.conf.Configured;
+import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.util.Tool;
 import org.apache.hadoop.util.ToolRunner;
 import org.apache.log4j.Level;
@@ -67,6 +72,16 @@ public class Main extends Configured implements Tool {
       description = "Print extra debugging information")
   private boolean debug = false;
 
+  @Parameter(
+      names = {"--conf", "--property"},
+      description = "Set a configuration property (format: key=value). Can be specified multiple times.")
+  private List<String> confProperties;
+
+  @Parameter(
+      names = {"--config-file"},
+      description = "Path to a configuration file (properties or Hadoop XML format).")
+  private String configFilePath;
+
   @VisibleForTesting
   @Parameter(names = "--dollar-zero", description = "A way for the runtime path to be passed in", hidden = true)
   String programName = DEFAULT_PROGRAM_NAME;
@@ -162,10 +177,43 @@ public class Main extends Configured implements Tool {
       return 1;
     }
 
-    try {
-      if (command instanceof Configurable) {
-        ((Configurable) command).setConf(getConf());
+    // Note to developer: This is a generic way to apply configs to given command.
+    // If the command does not support the configs, it would simply be ignored.
+    if (command instanceof Configurable) {
+      Configuration merged = new Configuration(getConf());
+
+      if (configFilePath != null) {
+        try {
+          if (isXmlConfigFile(configFilePath)) {
+            loadXmlConfiguration(merged, configFilePath);
+          } else if (isPropertiesConfigFile(configFilePath)) {
+            loadPropertiesConfiguration(merged, configFilePath);
+          } else {
+            throw new IllegalArgumentException(
+                "Unsupported config file format. Only .xml and .properties files are supported: "
+                    + configFilePath);
+          }
+        } catch (Exception e) {
+          throw new IllegalArgumentException(
+              "Failed to load config file '" + configFilePath + "': " + e.getMessage(), e);
+        }
       }
+
+      if (confProperties != null) {
+        for (String prop : confProperties) {
+          String[] parts = prop.split("=", 2);
+          if (parts.length != 2) {
+            throw new IllegalArgumentException(
+                "Configuration property must be in format key=value: " + prop);
+          }
+          merged.set(parts[0].trim(), parts[1].trim());
+          console.debug("Set configuration property: {}={}", parts[0].trim(), parts[1].trim());
+        }
+      }
+      ((Configurable) command).setConf(merged);
+    }
+
+    try {
       return command.run();
     } catch (IllegalArgumentException e) {
       if (debug) {
@@ -197,4 +245,27 @@ public class Main extends Configured implements Tool {
     int rc = ToolRunner.run(new Configuration(), new Main(console), args);
     System.exit(rc);
   }
+
+  private boolean isXmlConfigFile(String filePath) {
+    return filePath.toLowerCase().endsWith(".xml");
+  }
+
+  private boolean isPropertiesConfigFile(String filePath) {
+    String lowerPath = filePath.toLowerCase();
+    return lowerPath.endsWith(".properties");
+  }
+
+  private void loadXmlConfiguration(Configuration config, String filePath) {
+    config.addResource(new Path(filePath));
+    console.debug("Loaded XML configuration from file: {}", filePath);
+  }
+
+  private void loadPropertiesConfiguration(Configuration config, String filePath) throws Exception {
+    try (InputStream in = new FileInputStream(filePath)) {
+      Properties props = new Properties();
+      props.load(in);
+      props.forEach((key, value) -> config.set(key.toString(), value.toString()));
+      console.debug("Loaded properties configuration from file: {}", filePath);
+    }
+  }
 }
