@@ -22,17 +22,18 @@ import com.beust.jcommander.Parameter;
 import com.beust.jcommander.Parameters;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
+import java.io.IOException;
+import java.util.List;
 import org.apache.hadoop.fs.Path;
 import org.apache.parquet.cli.BaseCommand;
 import org.apache.parquet.hadoop.util.ColumnPruner;
 import org.slf4j.Logger;
 
-import java.io.IOException;
-import java.util.List;
-
-@Parameters(commandDescription="(Deprecated: will be removed in 2.0.0, use rewrite command instead) " +
-        "Prune column(s) in a Parquet file and save it to a new file. " +
-        "The columns left are not changed.")
+@Deprecated
+@Parameters(
+    commandDescription = "(Deprecated: will be removed in 2.0.0, use rewrite command instead) "
+        + "Prune column(s) in a Parquet file and save it to a new file. "
+        + "The columns left are not changed.")
 public class PruneColumnsCommand extends BaseCommand {
 
   public PruneColumnsCommand(Logger console) {
@@ -40,30 +41,29 @@ public class PruneColumnsCommand extends BaseCommand {
   }
 
   @Parameter(
-    names = {"-i", "--input"},
-    description = "<input parquet file path>",
-    required = true)
+      names = {"-i", "--input"},
+      description = "<input parquet file path>",
+      required = true)
   String input;
 
   @Parameter(
-    names = {"-o", "--output"},
-    description = "<output parquet file path>",
-    required = true)
+      names = {"-o", "--output"},
+      description = "<output parquet file path>",
+      required = true)
   String output;
 
   @Parameter(
-    names = {"-c", "--columns"},
-    description = "<columns to be pruned>",
-    required = true)
+      names = {"-c", "--columns"},
+      description = "<columns to be pruned>",
+      required = true)
   List<String> cols;
 
   @Override
   public int run() throws IOException {
-    Preconditions.checkArgument(input != null && output != null,
-      "Both input and output parquet file paths are required.");
+    Preconditions.checkArgument(
+        input != null && output != null, "Both input and output parquet file paths are required.");
 
-    Preconditions.checkArgument(cols != null && cols.size() > 0,
-      "columns cannot be null or empty");
+    Preconditions.checkArgument(cols != null && !cols.isEmpty(), "columns cannot be null or empty");
 
     Path inPath = new Path(input);
     Path outPath = new Path(output);
@@ -75,8 +75,7 @@ public class PruneColumnsCommand extends BaseCommand {
   @Override
   public List<String> getExamples() {
     return Lists.newArrayList(
-      "# Removes specified columns and write to a new Parquet file",
-      "-i input.parquet -o output.parquet -c col1_name"
-    );
+        "# Removes specified columns and write to a new Parquet file",
+        "-i input.parquet -o output.parquet -c col1_name");
   }
 }
