@@ -20,6 +20,10 @@
 package org.apache.parquet.cli.commands;
 
 import com.beust.jcommander.JCommander;
+import java.io.BufferedWriter;
+import java.io.File;
+import java.io.FileWriter;
+import java.io.IOException;
 import org.apache.commons.io.FileUtils;
 import org.apache.hadoop.fs.FileAlreadyExistsException;
 import org.junit.Assert;
@@ -27,11 +31,6 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
 
-import java.io.BufferedWriter;
-import java.io.File;
-import java.io.FileWriter;
-import java.io.IOException;
-
 public class ToAvroCommandTest extends AvroFileTest {
   @Rule
   public TemporaryFolder folder = new TemporaryFolder();
@@ -48,10 +47,9 @@ public class ToAvroCommandTest extends AvroFileTest {
     final File avroOutputFile = folder.newFile("sample.avro");
 
     // Write the json to the file, so we can read it again.
-    final String inputJson = "{\"id\": 1, \"name\": \"Alice\"}\n" +
-      "{\"id\": 2, \"name\": \"Bob\"}\n" +
-      "{\"id\": 3, \"name\": \"Carol\"}\n" +
-      "{\"id\": 4, \"name\": \"Dave\"}";
+    final String inputJson = "{\"id\": 1, \"name\": \"Alice\"}\n" + "{\"id\": 2, \"name\": \"Bob\"}\n"
+        + "{\"id\": 3, \"name\": \"Carol\"}\n"
+        + "{\"id\": 4, \"name\": \"Dave\"}";
 
     try (BufferedWriter writer = new BufferedWriter(new FileWriter(jsonInputFile))) {
       writer.write(inputJson);
@@ -59,20 +57,15 @@ public class ToAvroCommandTest extends AvroFileTest {
 
     ToAvroCommand cmd = new ToAvroCommand(null);
 
-    JCommander
-      .newBuilder()
-      .addObject(cmd)
-      .build()
-      .parse(
-        "--overwrite",
-        jsonInputFile.getAbsolutePath(),
-        "--output",
-        avroOutputFile.getAbsolutePath()
-      );
+    JCommander.newBuilder()
+        .addObject(cmd)
+        .build()
+        .parse("--overwrite", jsonInputFile.getAbsolutePath(), "--output", avroOutputFile.getAbsolutePath());
 
     assert (cmd.run() == 0);
   }
 
+  @Test
   public void testToAvroCommandWithGzipCompression() throws IOException {
     File avroFile = toAvro(parquetFile(), "GZIP");
     Assert.assertTrue(avroFile.exists());
@@ -90,6 +83,18 @@ public class ToAvroCommandTest extends AvroFileTest {
     Assert.assertTrue(avroFile.exists());
   }
 
+  @Test
+  public void testToAvroCommandWithBzip2Compression() throws IOException {
+    File avroFile = toAvro(parquetFile(), "bzip2");
+    Assert.assertTrue(avroFile.exists());
+  }
+
+  @Test
+  public void testToAvroCommandWithXzCompression() throws IOException {
+    File avroFile = toAvro(parquetFile(), "xz");
+    Assert.assertTrue(avroFile.exists());
+  }
+
   @Test(expected = IllegalArgumentException.class)
   public void testToAvroCommandWithInvalidCompression() throws IOException {
     toAvro(parquetFile(), "FOO");
