@@ -18,14 +18,16 @@
  */
 package org.apache.parquet.io;
 
+import it.unimi.dsi.fastutil.ints.IntArrayList;
+import it.unimi.dsi.fastutil.ints.IntIterator;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
-
 import org.apache.parquet.column.ColumnWriteStore;
 import org.apache.parquet.column.ColumnWriter;
 import org.apache.parquet.column.impl.ColumnReadStoreImpl;
@@ -45,9 +47,6 @@ import org.apache.parquet.io.api.Binary;
 import org.apache.parquet.io.api.RecordConsumer;
 import org.apache.parquet.io.api.RecordMaterializer;
 import org.apache.parquet.schema.MessageType;
-
-import it.unimi.dsi.fastutil.ints.IntArrayList;
-import it.unimi.dsi.fastutil.ints.IntIterator;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -75,29 +74,26 @@ public class MessageColumnIO extends GroupColumnIO {
     return super.getColumnNames();
   }
 
-  public <T> RecordReader<T> getRecordReader(PageReadStore columns,
-                                             RecordMaterializer<T> recordMaterializer) {
+  public <T> RecordReader<T> getRecordReader(PageReadStore columns, RecordMaterializer<T> recordMaterializer) {
     return getRecordReader(columns, recordMaterializer, FilterCompat.NOOP);
   }
 
   /**
-   * @param columns a page read store with the column data
+   * @param columns            a page read store with the column data
    * @param recordMaterializer a record materializer
-   * @param filter a record filter
-   * @param <T> the type of records returned by the reader
+   * @param filter             a record filter
+   * @param <T>                the type of records returned by the reader
    * @return a record reader
    * @deprecated use getRecordReader(PageReadStore, RecordMaterializer, Filter)
    */
   @Deprecated
-  public <T> RecordReader<T> getRecordReader(PageReadStore columns,
-                                             RecordMaterializer<T> recordMaterializer,
-                                             UnboundRecordFilter filter) {
+  public <T> RecordReader<T> getRecordReader(
+      PageReadStore columns, RecordMaterializer<T> recordMaterializer, UnboundRecordFilter filter) {
     return getRecordReader(columns, recordMaterializer, FilterCompat.get(filter));
   }
 
-  public <T> RecordReader<T> getRecordReader(final PageReadStore columns,
-                                             final RecordMaterializer<T> recordMaterializer,
-                                             final Filter filter) {
+  public <T> RecordReader<T> getRecordReader(
+      final PageReadStore columns, final RecordMaterializer<T> recordMaterializer, final Filter filter) {
     Objects.requireNonNull(columns, "columns cannot be null");
     Objects.requireNonNull(recordMaterializer, "recordMaterializer cannot be null");
     Objects.requireNonNull(filter, "filter cannot be null");
@@ -111,19 +107,18 @@ public class MessageColumnIO extends GroupColumnIO {
       public RecordReader<T> visit(FilterPredicateCompat filterPredicateCompat) {
 
         FilterPredicate predicate = filterPredicateCompat.getFilterPredicate();
-        IncrementallyUpdatedFilterPredicateBuilder builder = new IncrementallyUpdatedFilterPredicateBuilder(leaves);
+        IncrementallyUpdatedFilterPredicateBuilder builder =
+            new IncrementallyUpdatedFilterPredicateBuilder(leaves);
         IncrementallyUpdatedFilterPredicate streamingPredicate = builder.build(predicate);
         RecordMaterializer<T> filteringRecordMaterializer = new FilteringRecordMaterializer<T>(
-            recordMaterializer,
-            leaves,
-            builder.getValueInspectorsByColumn(),
-            streamingPredicate);
+            recordMaterializer, leaves, builder.getValueInspectorsByColumn(), streamingPredicate);
 
         return new RecordReaderImplementation<>(
             MessageColumnIO.this,
             filteringRecordMaterializer,
             validating,
-            new ColumnReadStoreImpl(columns, filteringRecordMaterializer.getRootConverter(), getType(), createdBy));
+            new ColumnReadStoreImpl(
+                columns, filteringRecordMaterializer.getRootConverter(), getType(), createdBy));
       }
 
       @Override
@@ -134,8 +129,7 @@ public class MessageColumnIO extends GroupColumnIO {
             validating,
             new ColumnReadStoreImpl(columns, recordMaterializer.getRootConverter(), getType(), createdBy),
             unboundRecordFilterCompat.getUnboundRecordFilter(),
-            columns.getRowCount()
-        );
+            columns.getRowCount());
       }
 
       @Override
@@ -152,34 +146,33 @@ public class MessageColumnIO extends GroupColumnIO {
   /**
    * To improve null writing performance, we cache null values on group nodes. We flush nulls when a
    * non-null value hits the group node.
-   *
+   * <p>
    * Intuitively, when a group node hits a null value, all the leaves underneath it should be null.
    * A direct way of doing it is to write nulls for all the leaves underneath it when a group node
    * is null. This approach is not optimal, consider following case:
-   *
-   *    - When the schema is really wide where for each group node, there are thousands of leaf
-   *    nodes underneath it.
-   *    - When the data being written is really sparse, group nodes could hit nulls frequently.
-   *
+   * <p>
+   * - When the schema is really wide where for each group node, there are thousands of leaf
+   * nodes underneath it.
+   * - When the data being written is really sparse, group nodes could hit nulls frequently.
+   * <p>
    * With the direct approach, if a group node hit null values a thousand times, and there are a
    * thousand nodes underneath it.
    * For each null value, it iterates over a thousand leaf writers to write null values and it
-   *  will do it for a thousand null values.
-   *
+   * will do it for a thousand null values.
+   * <p>
    * In the above case, each leaf writer maintains it's own buffer of values, calling thousands of
    * them in turn is very bad for memory locality. Instead each group node can remember the null values
    * encountered and flush only when a non-null value hits the group node. In this way, when we flush
    * null values, we only iterate through all the leaves 1 time and multiple cached null values are
    * flushed to each leaf in a tight loop. This implementation has following characteristics.
-   *
-   *    1. When a group node hits a null value, it adds the repetition level of the null value to
-   *    the groupNullCache. The definition level of the cached nulls should always be the same as
-   *    the definition level of the group node so there is no need to store it.
-   *
-   *    2. When a group node hits a non null value and it has null value cached, it should flush null
-   *    values and start from his children group nodes first. This make sure the order of null values
-   *     being flushed is correct.
-   *
+   * <p>
+   * 1. When a group node hits a null value, it adds the repetition level of the null value to
+   * the groupNullCache. The definition level of the cached nulls should always be the same as
+   * the definition level of the group node so there is no need to store it.
+   * <p>
+   * 2. When a group node hits a non null value and it has null value cached, it should flush null
+   * values and start from his children group nodes first. This make sure the order of null values
+   * being flushed is correct.
    */
   private class MessageColumnIORecordConsumer extends RecordConsumer {
     private ColumnIO currentColumnIO;
@@ -190,9 +183,7 @@ public class MessageColumnIO extends GroupColumnIO {
 
       @Override
       public String toString() {
-        return "VisitedIndex{" +
-            "visitedIndexes=" + visitedIndexes +
-            '}';
+        return "VisitedIndex{" + "visitedIndexes=" + visitedIndexes + '}';
       }
 
       public void reset(int fieldsCount) {
@@ -208,7 +199,7 @@ public class MessageColumnIO extends GroupColumnIO {
       }
     }
 
-    //track at each level of depth, which fields are written, so nulls can be inserted for the unwritten fields
+    // track at each level of depth, which fields are written, so nulls can be inserted for the unwritten fields
     private final FieldsMarker[] fieldsWritten;
     private final int[] r;
     private final ColumnWriter[] columnWriters;
@@ -219,7 +210,6 @@ public class MessageColumnIO extends GroupColumnIO {
      */
     private Map<GroupColumnIO, List<ColumnWriter>> groupToLeafWriter = new HashMap<>();
 
-
     /*
      * Cache nulls for each group node. It only stores the repetition level, since the definition level
      * should always be the definition level of the group node.
@@ -237,18 +227,14 @@ public class MessageColumnIO extends GroupColumnIO {
     }
 
     private List<ColumnWriter> getLeafWriters(GroupColumnIO group) {
-      List<ColumnWriter> writers = groupToLeafWriter.get(group);
-      if (writers == null) {
-        writers = new ArrayList<>();
-        groupToLeafWriter.put(group, writers);
-      }
-      return writers;
+      return groupToLeafWriter.computeIfAbsent(group, k -> new ArrayList<>());
     }
 
     public MessageColumnIORecordConsumer(ColumnWriteStore columns) {
       this.columns = columns;
       int maxDepth = 0;
-      this.columnWriters = new ColumnWriter[MessageColumnIO.this.getLeaves().size()];
+      this.columnWriters =
+          new ColumnWriter[MessageColumnIO.this.getLeaves().size()];
 
       for (PrimitiveColumnIO primitiveColumnIO : MessageColumnIO.this.getLeaves()) {
         ColumnWriter w = columns.getColumnWriter(primitiveColumnIO.getColumnDescriptor());
@@ -266,21 +252,27 @@ public class MessageColumnIO extends GroupColumnIO {
 
     private void printState() {
       if (DEBUG) {
-        log(currentLevel + ", " + fieldsWritten[currentLevel] + ": " + Arrays.toString(currentColumnIO.getFieldPath()) + " r:" + r[currentLevel]);
+        log(currentLevel + ", " + fieldsWritten[currentLevel] + ": "
+            + Arrays.toString(currentColumnIO.getFieldPath()) + " r:" + r[currentLevel]);
         if (r[currentLevel] > currentColumnIO.getRepetitionLevel()) {
           // sanity check
-          throw new InvalidRecordException(r[currentLevel] + "(r) > " + currentColumnIO.getRepetitionLevel() + " ( schema r)");
+          throw new InvalidRecordException(
+              r[currentLevel] + "(r) > " + currentColumnIO.getRepetitionLevel() + " ( schema r)");
         }
       }
     }
 
-    private void log(Object message, Object...parameters) {
+    private void log(Object message, Object... parameters) {
       if (DEBUG) {
         StringBuilder indent = new StringBuilder(currentLevel * 2);
         for (int i = 0; i < currentLevel; ++i) {
           indent.append("  ");
         }
-        LOG.debug(indent.toString() + message, parameters);
+        if (parameters.length == 0) {
+          LOG.debug(indent.toString() + message);
+        } else {
+          LOG.debug(indent.toString() + message, parameters);
+        }
       }
     }
 
@@ -323,10 +315,11 @@ public class MessageColumnIO extends GroupColumnIO {
 
     @Override
     public void endField(String field, int index) {
-      if (DEBUG) log("endField({}, {})",field ,index);
+      if (DEBUG) log("endField({}, {})", field, index);
       currentColumnIO = currentColumnIO.getParent();
       if (emptyField) {
-        throw new ParquetEncodingException("empty fields are illegal, the field should be ommited completely instead");
+        throw new ParquetEncodingException(
+            "empty fields are illegal, the field should be ommited completely instead");
       }
       fieldsWritten[currentLevel].markWritten(index);
       r[currentLevel] = currentLevel == 0 ? 0 : r[currentLevel - 1];
@@ -340,10 +333,15 @@ public class MessageColumnIO extends GroupColumnIO {
           try {
             ColumnIO undefinedField = ((GroupColumnIO) currentColumnIO).getChild(i);
             int d = currentColumnIO.getDefinitionLevel();
-            if (DEBUG) log(Arrays.toString(undefinedField.getFieldPath()) + ".writeNull(" + r[currentLevel] + "," + d + ")");
+            if (DEBUG)
+              log(Arrays.toString(undefinedField.getFieldPath()) + ".writeNull(" + r[currentLevel] + ","
+                  + d + ")");
             writeNull(undefinedField, r[currentLevel], d);
           } catch (RuntimeException e) {
-            throw new ParquetEncodingException("error while writing nulls for fields of indexes " + i + " . current index: " + fieldsWritten[currentLevel], e);
+            throw new ParquetEncodingException(
+                "error while writing nulls for fields of indexes " + i + " . current index: "
+                    + fieldsWritten[currentLevel],
+                e);
           }
         }
       }
@@ -354,7 +352,8 @@ public class MessageColumnIO extends GroupColumnIO {
         columnWriters[((PrimitiveColumnIO) undefinedField).getId()].writeNull(r, d);
       } else {
         GroupColumnIO groupColumnIO = (GroupColumnIO) undefinedField;
-        // only cache the repetition level, the definition level should always be the definition level of the parent node
+        // only cache the repetition level, the definition level should always be the definition level of the
+        // parent node
         cacheNullForGroup(groupColumnIO, r);
       }
     }
@@ -370,12 +369,11 @@ public class MessageColumnIO extends GroupColumnIO {
 
     private void writeNullToLeaves(GroupColumnIO group) {
       IntArrayList nullCache = groupNullCache.get(group);
-      if (nullCache == null || nullCache.isEmpty())
-        return;
+      if (nullCache == null || nullCache.isEmpty()) return;
 
       int parentDefinitionLevel = group.getParent().getDefinitionLevel();
       for (ColumnWriter leafWriter : groupToLeafWriter.get(group)) {
-        for (IntIterator iter = nullCache.iterator(); iter.hasNext();) {
+        for (IntIterator iter = nullCache.iterator(); iter.hasNext(); ) {
           int repetitionLevel = iter.nextInt();
           leafWriter.writeNull(repetitionLevel, parentDefinitionLevel);
         }
@@ -411,16 +409,15 @@ public class MessageColumnIO extends GroupColumnIO {
       return nulls != null && !nulls.isEmpty();
     }
 
-
     private void flushCachedNulls(GroupColumnIO group) {
-      //flush children first
+      // flush children first
       for (int i = 0; i < group.getChildrenCount(); i++) {
         ColumnIO child = group.getChild(i);
         if (child instanceof GroupColumnIO) {
           flushCachedNulls((GroupColumnIO) child);
         }
       }
-      //then flush itself
+      // then flush itself
       writeNullToLeaves(group);
     }
 
@@ -499,7 +496,6 @@ public class MessageColumnIO extends GroupColumnIO {
       if (DEBUG) printState();
     }
 
-
     /**
      * Flush null for all groups
      */
@@ -516,7 +512,7 @@ public class MessageColumnIO extends GroupColumnIO {
   }
 
   void setLevels() {
-    setLevels(0, 0, new String[0], new int[0], Arrays.<ColumnIO>asList(this), Arrays.<ColumnIO>asList(this));
+    setLevels(0, 0, new String[0], new int[0], Collections.singletonList(this), Collections.singletonList(this));
   }
 
   void setLeaves(List<PrimitiveColumnIO> leaves) {
