@@ -1,4 +1,4 @@
-/* 
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -6,9 +6,9 @@
  * to you under the Apache License, Version 2.0 (the
  * "License"); you may not use this file except in compliance
  * with the License.  You may obtain a copy of the License at
- * 
+ *
  *   http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing,
  * software distributed under the License is distributed on an
  * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
@@ -20,7 +20,6 @@ package org.apache.parquet.column.values;
 
 import java.io.IOException;
 import java.util.Random;
-
 import org.apache.commons.lang3.RandomStringUtils;
 import org.apache.parquet.bytes.ByteBufferInputStream;
 import org.apache.parquet.io.api.Binary;
@@ -39,36 +38,32 @@ public class Utils {
     }
     return samples;
   }
-  
-  public static void writeInts(ValuesWriter writer, int[] ints)
-      throws IOException {
-    for(int i=0; i < ints.length; i++) {
+
+  public static void writeInts(ValuesWriter writer, int[] ints) throws IOException {
+    for (int i = 0; i < ints.length; i++) {
       writer.writeInteger(ints[i]);
     }
   }
 
-  public static void writeData(ValuesWriter writer, String[] strings)
-      throws IOException {
-    for(int i=0; i < strings.length; i++) {
+  public static void writeData(ValuesWriter writer, String[] strings) throws IOException {
+    for (int i = 0; i < strings.length; i++) {
       writer.writeBytes(Binary.fromString(strings[i]));
     }
   }
 
-  public static Binary[] readData(ValuesReader reader, ByteBufferInputStream stream, int length)
-      throws IOException {
+  public static Binary[] readData(ValuesReader reader, ByteBufferInputStream stream, int length) throws IOException {
     Binary[] bins = new Binary[length];
     reader.initFromPage(length, stream);
-    for(int i=0; i < length; i++) {
+    for (int i = 0; i < length; i++) {
       bins[i] = reader.readBytes();
     }
     return bins;
   }
 
-  public static int[] readInts(ValuesReader reader, ByteBufferInputStream stream, int length)
-      throws IOException {
+  public static int[] readInts(ValuesReader reader, ByteBufferInputStream stream, int length) throws IOException {
     int[] ints = new int[length];
     reader.initFromPage(length, stream);
-    for(int i=0; i < length; i++) {
+    for (int i = 0; i < length; i++) {
       ints[i] = reader.readInteger();
     }
     return ints;
