@@ -18,20 +18,6 @@
  */
 package org.apache.parquet.filter2.predicate;
 
-import org.junit.Test;
-
-import org.apache.parquet.hadoop.metadata.ColumnPath;
-import org.apache.parquet.filter2.predicate.Operators.BinaryColumn;
-import org.apache.parquet.filter2.predicate.Operators.BooleanColumn;
-import org.apache.parquet.filter2.predicate.Operators.Column;
-import org.apache.parquet.filter2.predicate.Operators.DoubleColumn;
-import org.apache.parquet.filter2.predicate.Operators.FloatColumn;
-import org.apache.parquet.filter2.predicate.Operators.IntColumn;
-import org.apache.parquet.filter2.predicate.Operators.LongColumn;
-import org.apache.parquet.schema.PrimitiveType.PrimitiveTypeName;
-
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.fail;
 import static org.apache.parquet.filter2.predicate.FilterApi.binaryColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.booleanColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.doubleColumn;
@@ -39,6 +25,19 @@ import static org.apache.parquet.filter2.predicate.FilterApi.floatColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.intColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.longColumn;
 import static org.apache.parquet.filter2.predicate.ValidTypeMap.assertTypeValid;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.fail;
+
+import org.apache.parquet.filter2.predicate.Operators.BinaryColumn;
+import org.apache.parquet.filter2.predicate.Operators.BooleanColumn;
+import org.apache.parquet.filter2.predicate.Operators.Column;
+import org.apache.parquet.filter2.predicate.Operators.DoubleColumn;
+import org.apache.parquet.filter2.predicate.Operators.FloatColumn;
+import org.apache.parquet.filter2.predicate.Operators.IntColumn;
+import org.apache.parquet.filter2.predicate.Operators.LongColumn;
+import org.apache.parquet.hadoop.metadata.ColumnPath;
+import org.apache.parquet.schema.PrimitiveType.PrimitiveTypeName;
+import org.junit.Test;
 
 public class TestValidTypeMap {
   public static IntColumn intColumn = intColumn("int.column");
@@ -56,7 +55,7 @@ public class TestValidTypeMap {
   }
 
   public static Column<InvalidColumnType> invalidColumn =
-      new Column<InvalidColumnType>(ColumnPath.get("invalid.column"), InvalidColumnType.class) { };
+      new Column<InvalidColumnType>(ColumnPath.get("invalid.column"), InvalidColumnType.class) {};
 
   @Test
   public void testValidTypes() {
@@ -76,10 +75,12 @@ public class TestValidTypeMap {
       assertTypeValid(intColumn, PrimitiveTypeName.DOUBLE);
       fail("This should throw!");
     } catch (IllegalArgumentException e) {
-      assertEquals("FilterPredicate column: int.column's declared type (java.lang.Integer) does not match the "
-          + "schema found in file metadata. Column int.column is of type: "
-          + "DOUBLE\n"
-          + "Valid types for this column are: [class java.lang.Double]", e.getMessage());
+      assertEquals(
+          "FilterPredicate column: int.column's declared type (java.lang.Integer) does not match the "
+              + "schema found in file metadata. Column int.column is of type: "
+              + "DOUBLE\n"
+              + "Valid types for this column are: [class java.lang.Double]",
+          e.getMessage());
     }
   }
 
@@ -89,10 +90,11 @@ public class TestValidTypeMap {
       assertTypeValid(invalidColumn, PrimitiveTypeName.INT32);
       fail("This should throw!");
     } catch (IllegalArgumentException e) {
-      assertEquals("Column invalid.column was declared as type: "
-          + "org.apache.parquet.filter2.predicate.TestValidTypeMap$InvalidColumnType which is not supported "
-          + "in FilterPredicates. Supported types for this column are: [class java.lang.Integer]", e.getMessage());
+      assertEquals(
+          "Column invalid.column was declared as type: "
+              + "org.apache.parquet.filter2.predicate.TestValidTypeMap$InvalidColumnType which is not supported "
+              + "in FilterPredicates. Supported types for this column are: [class java.lang.Integer]",
+          e.getMessage());
     }
   }
-
 }
