@@ -170,7 +170,7 @@ public class ParquetInputFormat<T> extends FileInputFormat<Void, T> {
   /**
    * Default value of parquet.hadoop.vectored.io.enabled is {@value}.
    */
-  public static final boolean HADOOP_VECTORED_IO_DEFAULT = false;
+  public static final boolean HADOOP_VECTORED_IO_DEFAULT = true;
 
   public static void setTaskSideMetaData(Job job, boolean taskSideMetadata) {
     ContextUtil.getConfiguration(job).setBoolean(TASK_SIDE_METADATA, taskSideMetadata);
