@@ -508,7 +508,8 @@ public class ParquetRewriter implements Closeable {
           chunk.getValueCount(),
           chunk.getTotalSize(),
           chunk.getTotalUncompressedSize(),
-          chunk.getSizeStatistics());
+          chunk.getSizeStatistics(),
+          chunk.getGeospatialStatistics());
     }
 
     ColumnDescriptor descriptorOriginal = outSchema.getColumns().get(outColumnIdx);
@@ -762,6 +763,7 @@ public class ParquetRewriter implements Closeable {
               dlLevels,
               converter.getEncoding(headerV2.getEncoding()),
               BytesInput.from(pageLoad),
+              headerV2.is_compressed,
               rawDataLength,
               statistics,
               metaEncryptor,
