@@ -23,6 +23,9 @@ import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 
 import org.apache.hadoop.conf.Configuration;
+import org.apache.parquet.conf.ParquetConfiguration;
+import org.apache.parquet.conf.PlainParquetConfiguration;
+import org.apache.parquet.hadoop.util.ConfigurationUtil;
 import org.junit.Test;
 
 public class EncryptionPropertiesFactoryTest {
@@ -45,4 +48,24 @@ public class EncryptionPropertiesFactoryTest {
         encryptionProperties.getColumnProperties(SampleEncryptionPropertiesFactory.COL2),
         SampleEncryptionPropertiesFactory.COL2_ENCR_PROPERTIES);
   }
+
+  @Test
+  public void testLoadEncPropertiesFactoryParquetConfiguration() {
+    ParquetConfiguration conf = new PlainParquetConfiguration();
+    conf.set(
+        EncryptionPropertiesFactory.CRYPTO_FACTORY_CLASS_PROPERTY_NAME,
+        "org.apache.parquet.crypto.SampleEncryptionPropertiesFactory");
+
+    EncryptionPropertiesFactory encryptionPropertiesFactory = EncryptionPropertiesFactory.loadFactory(conf);
+    FileEncryptionProperties encryptionProperties = encryptionPropertiesFactory.getFileEncryptionProperties(
+        ConfigurationUtil.createHadoopConfiguration(conf), null, null);
+
+    assertArrayEquals(encryptionProperties.getFooterKey(), SampleEncryptionPropertiesFactory.FOOTER_KEY);
+    assertEquals(
+        encryptionProperties.getColumnProperties(SampleEncryptionPropertiesFactory.COL1),
+        SampleEncryptionPropertiesFactory.COL1_ENCR_PROPERTIES);
+    assertEquals(
+        encryptionProperties.getColumnProperties(SampleEncryptionPropertiesFactory.COL2),
+        SampleEncryptionPropertiesFactory.COL2_ENCR_PROPERTIES);
+  }
 }
