@@ -19,11 +19,14 @@
 
 package org.apache.parquet.hadoop;
 
+import static org.apache.parquet.filter2.predicate.FilterApi.and;
 import static org.apache.parquet.filter2.predicate.FilterApi.binaryColumn;
+import static org.apache.parquet.filter2.predicate.FilterApi.contains;
 import static org.apache.parquet.filter2.predicate.FilterApi.doubleColumn;
 import static org.apache.parquet.filter2.predicate.FilterApi.eq;
 import static org.apache.parquet.filter2.predicate.FilterApi.in;
 import static org.apache.parquet.filter2.predicate.FilterApi.longColumn;
+import static org.apache.parquet.filter2.predicate.FilterApi.or;
 import static org.apache.parquet.hadoop.ParquetFileWriter.Mode.OVERWRITE;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
@@ -40,6 +43,7 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Random;
 import java.util.Set;
 import java.util.function.Predicate;
@@ -120,7 +124,8 @@ public class TestBloomFiltering {
     List<PhoneBookWriter.User> users = new ArrayList<>();
     List<String> names = generateNames(rowCount);
     for (int i = 0; i < rowCount; ++i) {
-      users.add(new PhoneBookWriter.User(i, names.get(i), generatePhoneNumbers(), generateLocation(i, rowCount)));
+      users.add(
+          new PhoneBookWriter.User(i, names.get(i), generatePhoneNumbers(i), generateLocation(i, rowCount)));
     }
     return users;
   }
@@ -171,12 +176,13 @@ public class TestBloomFiltering {
       names.add("len");
     }
     for (int i = 0; i < rowCount; ++i) {
-      users.add(new PhoneBookWriter.User(i, names.get(i), generatePhoneNumbers(), generateLocation(i, rowCount)));
+      users.add(
+          new PhoneBookWriter.User(i, names.get(i), generatePhoneNumbers(i), generateLocation(i, rowCount)));
     }
     return users;
   }
 
-  private static List<PhoneBookWriter.PhoneNumber> generatePhoneNumbers() {
+  private static List<PhoneBookWriter.PhoneNumber> generatePhoneNumbers(int index) {
     int length = RANDOM.nextInt(5) - 1;
     if (length < 0) {
       return null;
@@ -184,8 +190,8 @@ public class TestBloomFiltering {
     List<PhoneBookWriter.PhoneNumber> phoneNumbers = new ArrayList<>(length);
     for (int i = 0; i < length; ++i) {
       // 6 digits numbers
-      long number = Math.abs(RANDOM.nextLong() % 900000) + 100000;
-      phoneNumbers.add(new PhoneBookWriter.PhoneNumber(number, PHONE_KINDS[RANDOM.nextInt(PHONE_KINDS.length)]));
+      phoneNumbers.add(
+          new PhoneBookWriter.PhoneNumber(500L % index, PHONE_KINDS[RANDOM.nextInt(PHONE_KINDS.length)]));
     }
     return phoneNumbers;
   }
@@ -318,12 +324,17 @@ public class TestBloomFiltering {
           .withBloomFilterEnabled("name", true)
           .withBloomFilterCandidateNumber("name", 10)
           .withBloomFilterEnabled("id", true)
-          .withBloomFilterCandidateNumber("id", 10);
+          .withBloomFilterCandidateNumber("id", 10)
+          .withDictionaryEncoding("phoneNumbers.phone.number", false)
+          .withBloomFilterEnabled("phoneNumbers.phone.number", true)
+          .withBloomFilterCandidateNumber("phoneNumbers.phone.number", 10);
     } else {
       writeBuilder
           .withBloomFilterNDV("location.lat", 10000L)
           .withBloomFilterNDV("name", 10000L)
-          .withBloomFilterNDV("id", 10000L);
+          .withBloomFilterNDV("id", 10000L)
+          .withDictionaryEncoding("phoneNumbers.phone.number", false)
+          .withBloomFilterNDV("phoneNumbers.phone.number", 10000L);
     }
     PhoneBookWriter.write(writeBuilder, DATA);
   }
@@ -398,6 +409,38 @@ public class TestBloomFiltering {
         eq(doubleColumn("location.lat"), 99.9));
   }
 
+  @Test
+  public void testContainsEqFiltering() throws IOException {
+    assertCorrectFiltering(
+        record -> Optional.ofNullable(record.getPhoneNumbers())
+            .map(numbers -> numbers.stream().anyMatch(n -> n.getNumber() == 250L))
+            .orElse(false),
+        contains(eq(longColumn("phoneNumbers.phone.number"), 250L)));
+  }
+
+  @Test
+  public void testContainsOrFiltering() throws IOException {
+    assertCorrectFiltering(
+        record -> Optional.ofNullable(record.getPhoneNumbers())
+            .map(numbers -> numbers.stream().anyMatch(n -> n.getNumber() == 250L || n.getNumber() == 50L))
+            .orElse(false),
+        or(
+            contains(eq(longColumn("phoneNumbers.phone.number"), 250L)),
+            contains(eq(longColumn("phoneNumbers.phone.number"), 50L))));
+  }
+
+  @Test
+  public void testContainsAndFiltering() throws IOException {
+    assertCorrectFiltering(
+        record -> Optional.ofNullable(record.getPhoneNumbers())
+            .map(numbers -> numbers.stream().anyMatch(n -> n.getNumber() == 10L)
+                && numbers.stream().anyMatch(n -> n.getNumber() == 5L))
+            .orElse(false),
+        and(
+            contains(eq(longColumn("phoneNumbers.phone.number"), 10L)),
+            contains(eq(longColumn("phoneNumbers.phone.number"), 5L))));
+  }
+
   @Test
   public void checkBloomFilterSize() throws IOException {
     FileDecryptionProperties fileDecryptionProperties = getFileDecryptionProperties();
