@@ -18,33 +18,41 @@
  */
 package org.apache.parquet.thrift;
 
+import static org.apache.parquet.schema.Type.Repetition.REPEATED;
+
 import com.twitter.elephantbird.thrift.TStructDescriptor;
 import com.twitter.elephantbird.thrift.TStructDescriptor.Field;
+import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
-
 import org.apache.hadoop.conf.Configuration;
-import org.apache.thrift.TBase;
-import org.apache.thrift.TEnum;
-import org.apache.thrift.TUnion;
-
-import org.apache.parquet.schema.Type;
+import org.apache.parquet.conf.HadoopParquetConfiguration;
+import org.apache.parquet.conf.ParquetConfiguration;
 import org.apache.parquet.schema.MessageType;
+import org.apache.parquet.schema.Type;
 import org.apache.parquet.thrift.projection.FieldProjectionFilter;
 import org.apache.parquet.thrift.struct.ThriftField;
 import org.apache.parquet.thrift.struct.ThriftField.Requirement;
 import org.apache.parquet.thrift.struct.ThriftType;
-import org.apache.parquet.thrift.struct.ThriftType.*;
+import org.apache.parquet.thrift.struct.ThriftType.BoolType;
+import org.apache.parquet.thrift.struct.ThriftType.ByteType;
+import org.apache.parquet.thrift.struct.ThriftType.DoubleType;
+import org.apache.parquet.thrift.struct.ThriftType.EnumType;
+import org.apache.parquet.thrift.struct.ThriftType.EnumValue;
+import org.apache.parquet.thrift.struct.ThriftType.I16Type;
+import org.apache.parquet.thrift.struct.ThriftType.I32Type;
+import org.apache.parquet.thrift.struct.ThriftType.I64Type;
+import org.apache.parquet.thrift.struct.ThriftType.StringType;
+import org.apache.parquet.thrift.struct.ThriftType.StructType;
 import org.apache.parquet.thrift.struct.ThriftType.StructType.StructOrUnionType;
 import org.apache.parquet.thrift.struct.ThriftTypeID;
+import org.apache.thrift.TBase;
+import org.apache.thrift.TEnum;
+import org.apache.thrift.TUnion;
 import org.apache.thrift.meta_data.FieldMetaData;
 
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.List;
-
-import static org.apache.parquet.schema.Type.Repetition.REPEATED;
-
 /**
  * Given a thrift class, this class converts it to parquet schema,
  * a {@link FieldProjectionFilter} can be specified for projection pushdown.
@@ -52,19 +60,26 @@ import static org.apache.parquet.schema.Type.Repetition.REPEATED;
 public class ThriftSchemaConverter {
   private final FieldProjectionFilter fieldProjectionFilter;
 
-  private Configuration conf;
+  private ParquetConfiguration conf;
 
   public ThriftSchemaConverter() {
     this(FieldProjectionFilter.ALL_COLUMNS);
   }
 
   public ThriftSchemaConverter(Configuration configuration) {
+    this(new HadoopParquetConfiguration(configuration));
+  }
+
+  public ThriftSchemaConverter(ParquetConfiguration configuration) {
     this();
     conf = configuration;
   }
 
-  public ThriftSchemaConverter(
-      Configuration configuration, FieldProjectionFilter fieldProjectionFilter) {
+  public ThriftSchemaConverter(Configuration configuration, FieldProjectionFilter fieldProjectionFilter) {
+    this(new HadoopParquetConfiguration(configuration), fieldProjectionFilter);
+  }
+
+  public ThriftSchemaConverter(ParquetConfiguration configuration, FieldProjectionFilter fieldProjectionFilter) {
     this(fieldProjectionFilter);
     conf = configuration;
   }
@@ -80,7 +95,7 @@ public class ThriftSchemaConverter {
   /**
    * struct is assumed to contain valid structOrUnionType metadata when used with this method.
    * This method may throw if structOrUnionType is unknown.
-   *
+   * <p>
    * Use convertWithoutProjection below to convert a StructType to MessageType
    *
    * @param struct the thrift type descriptor
@@ -100,10 +115,11 @@ public class ThriftSchemaConverter {
    * @return the struct as a Parquet message type
    */
   public static MessageType convertWithoutProjection(StructType struct) {
-    return ThriftSchemaConvertVisitor.convert(struct, FieldProjectionFilter.ALL_COLUMNS, false, new Configuration());
+    return ThriftSchemaConvertVisitor.convert(
+        struct, FieldProjectionFilter.ALL_COLUMNS, false, new Configuration());
   }
 
-  public static <T extends TBase<?,?>> StructOrUnionType structOrUnionType(Class<T> klass) {
+  public static <T extends TBase<?, ?>> StructOrUnionType structOrUnionType(Class<T> klass) {
     return TUnion.class.isAssignableFrom(klass) ? StructOrUnionType.UNION : StructOrUnionType.STRUCT;
   }
 
@@ -116,10 +132,9 @@ public class ThriftSchemaConverter {
     List<Field> fields = struct.getFields();
     List<ThriftField> children = new ArrayList<ThriftField>(fields.size());
     for (Field field : fields) {
-      Requirement req =
-          field.getFieldMetaData() == null ?
-              Requirement.OPTIONAL :
-              Requirement.fromType(field.getFieldMetaData().requirementType);
+      Requirement req = field.getFieldMetaData() == null
+          ? Requirement.OPTIONAL
+          : Requirement.fromType(field.getFieldMetaData().requirementType);
       children.add(toThriftField(field.getName(), field, req));
     }
     return new StructType(children, structOrUnionType(struct.getThriftClass()));
@@ -134,15 +149,14 @@ public class ThriftSchemaConverter {
    * <p>
    * This method never guesses because the expected ThriftField is known.
    *
-   * @param repeatedType a type that may be the element type
+   * @param repeatedType  a type that may be the element type
    * @param thriftElement the expected Schema for list elements
    * @return {@code true} if the repeatedType is the element schema
    */
-  static boolean isListElementType(Type repeatedType,
-                                   ThriftField thriftElement) {
-    if (repeatedType.isPrimitive() ||
-        (repeatedType.asGroupType().getFieldCount() != 1) ||
-        (repeatedType.asGroupType().getType(0).isRepetition(REPEATED))) {
+  static boolean isListElementType(Type repeatedType, ThriftField thriftElement) {
+    if (repeatedType.isPrimitive()
+        || (repeatedType.asGroupType().getFieldCount() != 1)
+        || (repeatedType.asGroupType().getType(0).isRepetition(REPEATED))) {
       // The repeated type must be the element type because it is an invalid
       // synthetic wrapper. Must be a group with one optional or required field
       return true;
@@ -211,16 +225,19 @@ public class ThriftSchemaConverter {
         final Field listElemField = field.getListElemField();
         type = new ThriftType.ListType(toThriftField(listElemField.getName(), listElemField, requirement));
         break;
+      case UUID:
       case ENUM:
-        Collection<TEnum> enumValues = field.getEnumValues();
-        List<EnumValue> values = new ArrayList<ThriftType.EnumValue>();
-        for (TEnum tEnum : enumValues) {
-          values.add(new EnumValue(tEnum.getValue(), tEnum.toString()));
+        if (field.isEnum()) {
+          Collection<TEnum> enumValues = field.getEnumValues();
+          List<ThriftType.EnumValue> values = new ArrayList<>();
+          for (TEnum tEnum : enumValues) {
+            values.add(new EnumValue(tEnum.getValue(), tEnum.toString()));
+          }
+          type = new EnumType(values);
+        } else {
+          type = new ThriftType.UUIDType();
         }
-        type = new EnumType(values);
-        break;
     }
     return new ThriftField(name, field.getId(), requirement, type);
   }
 }
-
